#include <stdio.h>
void main()
{
	printf("\n"
	"0x0 No error\n"
	"0x1 OP field of the configuration command error\n"
	"0x2 FC field of the configuration command error\n"
	"0x3 DL field of the configuration command error\n"
	"0x4 Fail to write data into device\n\n"
	"0x10001 Invalid device\n"
	"0x10002 Device already in used\n"
	"0x10003 Device not exist\n"
	"0x10004 Get device information error\n"
	"0x10005 Invalid USB package size\n"
	"0x10006 Write file fail\n"
	"0x10007 USB Tx buffer overflow\n"
	"0x10008 Error open file\n"
	"0x1000A Exceed maximum supported USB device\n"
	"0x1000B USB device not open\n"
	"0x1000C CreateRxTxThread fail\n"
	"0x1000D RestartRxThread fail\n"
	"0x1000E RestartTxThread fail\n"
	"0x10100 Communication timeout\n"
	"0x10101 Invalid CAN port number\n"
	"0x10102 No data in CAN received buffer\n"
	"0x10103 CAN transmitted buffer overflow\n"
	"0x10104 CAN bit rate parameters not support\n"
	"0x10105 CAN Filter ID list size parameter not support\n");

}
